@@ -295,7 +295,7 @@ public:
                    Matrix<double> &Tareas) const;
   void calcQblocksAni(SparseMatrix<double> &G1, const Matrix<double> &gamma,
                       const Matrix<double> &vec) const;
-  void calcGradientMatrices(SparseMatrix<double> **D) const;
+  std::vector<SparseMatrix<double>> calcGradientMatrices() const;
 
   // No need for IOHeader and IOHelper classes when using Rcpp
 #ifndef FMESHER_WITH_R
@@ -312,7 +312,7 @@ public:
 #endif // not FMESHER_WITH_R
 };
 
-Matrix3double *make_globe_points(int subsegments, double radius);
+std::unique_ptr<Matrix<double>> make_globe_points(int subsegments, double radius);
 
 class MOAint {
   friend std::ostream &operator<<(std::ostream &output, const MOAint &MO);
